#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netdb.h>
#include "common.h"

char *cs_error_to_string (cs_error_t error)
{
#define CASE_ERROR(a) case a: return #a
  switch (error) {
    CASE_ERROR(CS_OK);
    CASE_ERROR(CS_ERR_LIBRARY);
    CASE_ERROR(CS_ERR_VERSION);
    CASE_ERROR(CS_ERR_INIT);
    CASE_ERROR(CS_ERR_TIMEOUT);
    CASE_ERROR(CS_ERR_TRY_AGAIN);
    CASE_ERROR(CS_ERR_INVALID_PARAM);
    CASE_ERROR(CS_ERR_NO_MEMORY);
    CASE_ERROR(CS_ERR_BAD_HANDLE);
    CASE_ERROR(CS_ERR_BUSY);
    CASE_ERROR(CS_ERR_ACCESS);
    CASE_ERROR(CS_ERR_NOT_EXIST);
    CASE_ERROR(CS_ERR_NAME_TOO_LONG);
    CASE_ERROR(CS_ERR_EXIST);
    CASE_ERROR(CS_ERR_NO_SPACE);
    CASE_ERROR(CS_ERR_INTERRUPT);
    CASE_ERROR(CS_ERR_NAME_NOT_FOUND);
    CASE_ERROR(CS_ERR_NO_RESOURCES);
    CASE_ERROR(CS_ERR_NOT_SUPPORTED);
    CASE_ERROR(CS_ERR_BAD_OPERATION);
    CASE_ERROR(CS_ERR_FAILED_OPERATION);
    CASE_ERROR(CS_ERR_MESSAGE_ERROR);
    CASE_ERROR(CS_ERR_QUEUE_FULL);
    CASE_ERROR(CS_ERR_QUEUE_NOT_AVAILABLE);
    CASE_ERROR(CS_ERR_BAD_FLAGS);
    CASE_ERROR(CS_ERR_TOO_BIG);
    CASE_ERROR(CS_ERR_NO_SECTIONS);
    CASE_ERROR(CS_ERR_CONTEXT_NOT_FOUND);
    CASE_ERROR(CS_ERR_TOO_MANY_GROUPS);
    CASE_ERROR(CS_ERR_SECURITY);
  }

  return "<unknown corosync error>";
}

/* get at most 16 ip addresses of a given node */
void make_node_addresses (corosync_cfg_handle_t cfg_handle, int nodeid,
                          char *buffer, size_t buffer_size)
{
  cs_error_t error;
  corosync_cfg_node_address_t addrs[MAX_NODE_ADDRESSES];
  int num_addrs;
  int i;
  char *p = buffer;
  size_t l = buffer_size;

  error = corosync_cfg_get_node_addrs(cfg_handle, nodeid,
                                      sizeof(addrs)/sizeof(addrs[0]),
                                      &num_addrs, addrs);
  if (error != CS_OK) {
    strncpy(buffer, "?", buffer_size);
    return;
  }

  for (i = 0; i < num_addrs; i++) {
    if (i > 0) {
      strncpy(p, " ", l);
      p++;
      l--;
    }

    if (getnameinfo((struct sockaddr*) addrs[i].address,
                    addrs[i].address_length, p, l, NULL, 0, NI_NUMERICHOST)) {
      strncpy(p, "?", l);
      p++;
      l--;
    } else {
      p = buffer + strlen(buffer);
      l = buffer_size - strlen(buffer);
    }
  }
}
